# Get all constants used for the target capital
getConstantModule <- function(expectedFinancialResultTable, expectedFinancialResultFactor, expectedInsuranceResult, RTK_goingConcern, RTK_runOff, additionalEffectsOnTC, daughterValue){
  
  # Expected financial results
  expectedFinancialResult <- expectedFinancialResultTable[, sum(return*exposure)] * expectedFinancialResultFactor
  
  # Additional effects on the target capital including delta RBC
  additionalEffects <-  getAdditionalEffectsOnTC(RTK_goingConcern, RTK_runOff, additionalEffectsOnTC, daughterValue)
  
  x <- list(expectedFinancialResult = expectedFinancialResult,
            expectedInsuranceResult = expectedInsuranceResult,
            additionalEffects = additionalEffects)
  retClass(x, "constantModule")
}

# Create a table with constants
#' @export
simulate.constantModule <- function(object, nsim, seed, ...){
  # Additional effects are on purpose with a negative sign as they increase the target capital (left tail)
  additionalEffects <- -getValue(object, "additionalEffects")
  # Expected results (no sign change needed)
  expectedFinancialResult <- getValue(object, "expectedFinancialResult")
  expectedInsuranceResult <- getValue(object, "expectedInsuranceResult")
  
  all <- expectedFinancialResult + expectedInsuranceResult + additionalEffects
  
  sim <- data.table(all = rep(all, nsim), expectedFinancialResult = expectedFinancialResult, expectedInsuranceResult = expectedInsuranceResult, additionalEffects = additionalEffects)
  
  result <- list(sim = sim, RF = NULL)
  return(result)
}

# Additional effects on the RBC
getAdditionalEffectsOnTC <- function(RTK_goingConcern, RTK_runOff, additionalEffectsOnTC, daughterValue){
  if(!is.na(daughterValue)){
    # No additional effects on TC from the daughter company
    return(0)
  }else{
    # Additional effects on the TC
    return(RTK_goingConcern - RTK_runOff + additionalEffectsOnTC)
  }
}