context("Deterministic")

initialIinterest <- data.table(currency = c("CHF"),
                               rowNumber = 6:8,
                               time = 1:3,
                               value = 0
)


test_that("getAdditionalEffectsOnTC", {
  # Case without daughtert company
  expect_identical(getAdditionalEffectsOnTC(100, 90, 1, NA_real_), 11)
  # Case wit daughtert company
  expect_identical(getAdditionalEffectsOnTC(90, 90, 0, 12345), 0)
})



test_that("getFX_to_CHF", {
  initialFX <- data.table(from = c("EUR", "USD"), to = c("CHF", "JPY"), fx = c(1.2, 2.2))
  expect_identical(getFX_to_CHF(initialFX, "EUR"), 1.2)
  expect_identical(getFX_to_CHF(initialFX, "CHF"), 1)
})




test_that("calculateMVMLife", {
  scalingFactorNormal <- 0.5
  mvm_cf1 <- data.table(name = "Mortality",
                        rowNumber = 7,
                        time = c(0:2),
                        value = 1
  )
  mvm_cf2 <- data.table(name = "Longevity",
                        rowNumber = 8,
                        time = c(0:2),
                        value = 2
  )
  MVM_CFLife <- MVM_CashFlowsLife <- rbindlist(list(mvm_cf1, mvm_cf2))
  refCurrency <- "CHF"
  volatility <- c(Mortality = 1, Longevity = 1)
  corrLife <- matrix(c(1,0,0,1), ncol=2)
  rownames(corrLife) <- c("Mortality", "Longevity")
  colnames(corrLife) <- c("Mortality", "Longevity")
  coc <- 0.5 # cost of capital
  
  initialIR <- initialIinterest[currency == refCurrency]
  initialIR[, discount := exp(-time * value)]
  initialIR <- initialIR[, .(time = c(0, time), discount = c(1, discount))]
  lastDiscount <- initialIR[time == 3, discount]
  
  MVM_CFLife[initialIR, discount := i.discount, on = "time"]
  setorder(MVM_CFLife, -"time")
  MVM_CFLife[, alpha_value := cumsum(value * discount) / (sum(value * discount) * discount), by = "name"]
  MVM_sigma <- MVM_CFLife[, .(sigma = sqrt(as.numeric(t(alpha_value * volatility[name]) %*% corrLife[name, name] %*% (alpha_value * volatility[name]) )), discount = first(discount)), by = "time"]
  setorder(MVM_sigma, "time")
  MVM_EK <- MVM_sigma[, .(time = time + 1, EK = sigma * scalingFactorNormal, discount = c(discount[-1], lastDiscount))]
  MVM_Life <- coc * sum(MVM_EK$discount * MVM_EK$EK)
  
  y <- calculateMVMLife(MVM_CashFlowsLife, refCurrency, initialIinterest, volatility, corrLife, coc, alpha = 0.6977404152)
  expect_equal(MVM_Life, y, tolerance=1e-10)
})






# test_that("calculateValues", {
#   scenarios <- list()
#   Objects <- list(TC_additionalEffects = 1, expectedInsuranceResult=2, expectedFinancialResult=2, RTK_goingConcern=25,
#                   MVMbeforeNhmr = 5, param_nhmrFactor=0, FX_to_CHF=1.3)
#   ObjectsList <- list(Objects)
#   packageEnv$param <- list(standaloneTypesDiversification = c("spread", "interest"))
#   ES <- list(insurance_market_credit.all=12, market.all=3.5, insurance.all=4, credit.all=1, market.spread=1.5, market.interest=1.5,
#              insurance_market_credit_scenario.all = 12)
#   values <- list()
#   values[["additionalEffectOnTC"]] <- Objects$TC_additionalEffects
#   values[["expectedInsuranceResult"]] <- sum(getMultipleValues(ObjectsList, "expectedInsuranceResult"))
#   values[["expectedFinancialResult"]] <- sum(getMultipleValues(ObjectsList, "expectedFinancialResult"))
#   values[["RTK"]] <- Objects$RTK_goingConcern
#   values[["diversificationMarketInsuranceCredit"]] <- ES$insurance_market_credit.all - ES$market.all - ES$insurance.all - ES$credit.all
#   values[["market.diversification"]] <- ES$market.all - sum(getValue(ES, paste0("market.", getParam("standaloneTypesDiversification"))))
#   values[["SCR"]] <- ES$insurance_market_credit_scenario.all - values$expectedInsuranceResult - values$expectedFinancialResult + values$additionalEffectOnTC
#   values[["MVM"]] <- Objects$MVMbeforeNhmr + Objects$param_nhmrFactor * ES$market.all
#   values[["scenarioImpact"]] <- ES$insurance_market_credit_scenario.all - ES$insurance_market_credit.all
#   values[["targetCapital"]] <- values$SCR + values$MVM
#   values[["SSTRatio"]] <- (values$RTK - values$MVM)/(values$targetCapital - values$MVM)
#   values[["FX_to_CHF"]] <- Objects$FX_to_CHF
#   values <- c(values, ES, scenarios)
#   
#   out <- calculateValues(ES, ObjectsList, scenarios)
#   expect_identical(out[sort(names(out))], values[sort(names(values))])
# })




