context("Get")


test_that("getTargetCurrency", {
  marketRiskFactorsCurrency <- data.table(currency = c("EUR", "USD", "GBP", "JPY", "CHF"), targetCurrency = c("CHF", "CHF", "CHF", "USD", "CHF"), rowNumber = 1:5)
  # Classical case
  expect_identical(getTargetCurrency(marketRiskFactorsCurrency[1:3]), "CHF")
  expect_identical(getTargetCurrency(marketRiskFactorsCurrency[4]), "USD")
  
  # Incorrect case
  expect_error(getTargetCurrency(marketRiskFactorsCurrency[2:4]), "There should be exactly one target currency")
  expect_error(getTargetCurrency(marketRiskFactorsCurrency[c(1,2,5)]), "The currency should not be identical to the target currency")
})


# For 'getTable' checks
Template <- list(A = matrix(c(NA, NA, NA, NA, "A", "B", NA, "C", "D", NA, NA, "E", "F", "G", NA), nrow = 3))

test_that("getTable (table)",{
  
  # Classical case
  configTable <- list(value = list(sheet = "A", row = 2, col = 2:3), columns = list(Name = c("x", "y")), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "test")
  result <- data.table(x = c("A", "B"), y = c("C", "D"), rowNumber = 2:3)
  expect_identical(getTable(configTable, Template), result)
  
  # One column only
  configTable <- list(value = list(sheet = "A", row = 1, col = 5), columns = list(Name = c("x")), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "test")
  result <- data.table(x = c("F", "G", NA), rowNumber = 1:3)
  expect_identical(getTable(configTable, Template), result)
  
  # Correlation matrix classical case (ensure NA value in first column is ignored)
  configTable <- list(value = list(sheet = "A", row = 3, col = 1:10), columns = list(Name = LETTERS[1:10]), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "correlationMarket")
  result <- data.table(A = NA_character_, B = "B", C = "D", rowNumber = 3L)
  expect_identical(getTable(configTable, Template), result)
  
  # Correlation matrix
  configTable <- list(value = list(sheet = "A", row = 3, col = 2:10), columns = list(Name = LETTERS[2:10]), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "correlationMarket")
  result <- data.table(B = "B", C = "D", rowNumber = 3L)
  expect_identical(getTable(configTable, Template), result)
  
  # Correlation matrix (degenerate case)
  configTable <- list(value = list(sheet = "A", row = 3, col = 3:10), columns = list(Name = LETTERS[3:10]), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "correlationMarket")
  result <- data.table(C = "D", rowNumber = 3L)
  expect_identical(getTable(configTable, Template), result)
})


test_that("getTable (value)", {
  # One value
  configTable <- list(value = list(sheet = "A", row = 3, col = 4), columns = list(Name = c("x")), indicator = list(isValue = TRUE, isCorrelation = FALSE), keyword = "test")
  result <- data.table(x = "E", rowNumber = 1)
  expect_identical(getTable(configTable, Template), result)
  
  # One value outside of range
  configTable <- list(value = list(sheet = "A", row = 30, col = 4), columns = list(Name = c("x")), indicator = list(isValue = TRUE, isCorrelation = FALSE), keyword = "test")
  result <- data.table(x = NA_character_, rowNumber = 1)
  expect_identical(getTable(configTable, Template), result)
  
})

test_that("getTable (error handling)",{
  # Column outside of template
  configTable <- list(value = list(sheet = "A", row = 2, col = 5:6), columns = list(Name = c("x", "y")), indicator = list(isValue = FALSE, isCorrelation = FALSE), keyword = "test")
  result <- data.table(x = c("A", "B"), y = c("C", "D"), rowNumber = 2:3)
  expect_error(getTable(configTable, Template), "The column should not be empty")
  
  # Column outside of template for correlation matrix
  configTable <- list(value = list(sheet = "A", row = 2, col = 5:6), columns = list(Name = c("x", "y")), indicator = list(isValue = FALSE, isCorrelation = TRUE), keyword = "test")
  result <- data.table(x = c("A", "B"), y = c("C", "D"), rowNumber = 2:3)
  expect_error(getTable(configTable, Template), "The column 'y' is missing in the correlation matrix")
  })


test_that("getParam",{
  cleanup()
  expect_identical(getParam("riskTypes"), packageEnv$param$riskTypes)
  expect_identical(getParam("alpha"), 0.01)
  expect_error(getParam("xxxx"), "The object 'xxxx' is missing")
})







# test_that("getDependency",{
#   correlation <- matrix(c(1,0.1,0.1,1), ncol=2)
#   # Without scenarios
#   x <- list(mu = rep(0, ncol(correlation)), Sigma = correlation)
#   expect_identical(getDependency(correlation), retClass(x, "Reordering"))
#   
#   # With scenarios
#   n <- ncol(correlation)
#   corr <- matrix(c(1,0.1,0,0, 0.1,1,0,0, 0,0,1,0, 0,0,0,1), ncol=4, byrow=TRUE)
#   rownames(corr) <- c("", "", "scenario", "constant")
#   colnames(corr) <- c("", "", "scenario", "constant")
#   x <- list(mu = rep(0, n+2), Sigma = corr)
#   expect_identical(getDependency(correlation, TRUE), retClass(x, "Reordering"))
# })

  
  
#   riskTypes <- setNames(nm = c("market", "participation", "life"))
#   
#   result <- data.table(market.all = c(7,5,11), market.equity = c(0,2,1), market.participation = c(4,3,10), life.all = c(3, 2, 1))
#   expect_identical(getReorderedSimulations(sim, ordering, riskTypes), result) 
# })

# test_that("getAggregatedSimulations", {
#   simOriginal <- data.table(life.all = 1:3, nonlife.all = c(10, 20, 40), health.all = c(1, 10, 20), market.all = c(5, 1, 10), scenario.all = c(-100, 0, 0))
#   sim <- copy(simOriginal)
#   sim1 <- copy(simOriginal)
#   sim1[, insurance.all := c(12, 32, 63)][, insurance_market.all := c(17, 33, 73)][, insurance_market_scenario.all := c(-83, 33,73)][, simulation_id := 1:3]
#   
#   getAggregatedSimulations(sim)
#   expect_identical(sim, sim1)
# })

# test_that("getCreditRisk",{
#   creditRiskExposure <- data.table(weightedExposure = c(10, 20, 30, 100))
#   expect_identical(getCreditRisk(creditRiskExposure, creditRiskExposure[2], 0.1), 14)
#   expect_identical(getCreditRisk(creditRiskExposure, creditRiskExposure[integer(0)], 0.1), 16)
#   expect_identical(getCreditRisk(creditRiskExposure, creditRiskExposure[1:4], 0.1), 0)
# })

# test_that("performSimRiskAsParticipation", {
#   simTableOriginal <- data.table(market.participation = c(1, 2, 3), market.all = c(10, 20, 30), life.all = c(5, 5, 5), health.all = c(1, 2, 3), insurance_market.all = c(15, 25, 35), insurance_market_scenario.all = c(26, 37, 48))
#   
#   # One standalone
#   simTable <- copy(simTableOriginal)
#   simTableResult <- simTableOriginal[, .(market.participation = insurance_market_scenario.all, market.all = insurance_market_scenario.all, life.all = 0, health.all = 0, insurance_market.all = insurance_market_scenario.all, insurance_market_scenario.all)]
#   performSimRiskAsParticipation(simTable, riskAsParticipation = "Yes")
#   expect_identical(simTable, simTableResult)
#   
#   
#   # No impact
#   simTable <- copy(simTableOriginal)
#   performSimRiskAsParticipation(simTable, riskAsParticipation = "")
#   expect_identical(simTable, simTableOriginal)
#   
#   # No impact
#   simTable <- copy(simTableOriginal)
#   performSimRiskAsParticipation(simTable, riskAsParticipation = "No")
#   expect_identical(simTable, simTableOriginal)
# })

# test_that("performSimFlooring", {
#   simTableOriginal <- data.table(scenario.all = c(-1,0,0,0), insurance_market_scenario.all = c(-2, -3, 1, 5))
#   simTableResult <- data.table(scenario.all = c(1,3,0,0), insurance_market_scenario.all = c(0, 0, 1, 5))
#   
#   # Flooring correctly applied
#   sim <- copy(simTableOriginal)
#   performSimFlooring(sim, 0)
#   expect_identical(sim, simTableResult[, .(scenario.all, insurance_market_scenario.all, flooringThreshold = 0)])
# 
#   # No change when no flooring
#   sim <- copy(simTableOriginal)
#   performSimFlooring(sim, NA)
#   expect_identical(sim, simTableOriginal[, .(scenario.all, insurance_market_scenario.all, flooringThreshold = NA_real_)])
#   
#   # High treshold
#   sim <- copy(simTableOriginal)
#   newTable <- data.table(scenario.all = c(11, 13, 9, 5), insurance_market_scenario.all = c(10, 10, 10, 10), flooringThreshold = 10)
#   
#   performSimFlooring(sim, 10)
#   expect_identical(newTable, sim)
#   
#   # Low threshold
#   sim <- copy(simTableOriginal)
#   performSimFlooring(sim, -10)
#   expect_identical(sim, sim[, .(scenario.all, insurance_market_scenario.all, flooringThreshold = -10)])
# })




test_that("getMarketCovariance", {
  correlation <- matrix(c(1, 0.1, 0.1, 1), 2, 2, dimnames = list(c("a", "b"), c("a", "b")))
  volatility <- c(a=2, b= 5)
  expect_identical(unname(getMarketCovariance(correlation, volatility)), diag(volatility) %*% correlation %*% diag(volatility))
})


test_that("sumListDataTable", {
  A <- list(X = data.table(a = 1:10, b = 1:5),
            Y = data.table(a = 2:11, b = 1:5),
            Z = data.table(a = 3:12, b = 1:5))
  B <- data.table(a = as.integer((1:10)*3 + 3), b = as.integer((1:5)*3))
  expect_identical(sumListDataTable(A), B)
})


test_that("getLifeRisks", {
  correlation <- matrix(c(1, 0.1, 0.1, 1), 2, 2, dimnames = list(c("a", "b"), c("a", "b")))
  exposureLife <- c(a=2, b= 5)
  result <- getLifeRisks(correlation, exposureLife)
  expect_identical(result$correlation, correlation)
  expect_identical(result$volatility, exposureLife/stats::qnorm(0.995))
})

test_that("getHealthRisks", {
  correlation <- matrix(c(1, 0.1, 0.1, 1), 2, 2, dimnames = list(c("a", "b"), c("a", "b")))
  sensitivityHealth <- c(a=2, b=5)
  
  scenarioHealth <- data.table(label = c("A", "B", "none"), effect = c(1,3, 0), probability = c(0.1, 0.2, 0.7))
  expect_identical(getCompleteScenariosSet(keyword = NA, scenarioHealth[-3]), scenarioHealth)

  result <- getHealthRisks(correlation, sensitivityHealth, scenarioHealth[-3])
  expect_identical(result, retClass(list(correlation = correlation, volatility = sensitivityHealth, scenario = scenarioHealth), "healthModule"))
})



test_that("getRateWithTime",{

  marketRiskFactorsRate <- data.table(name = c("CHF 2Y", "CHF 10Y", "CHF 30Y"), horizon = c("k", "m", "l"))
  maturitiesInterestRate <- data.table(time = 1:50, projection = c(rep("k", 5), rep("m", 14), rep("l", 31)))
  
  RateWithTime_result <- data.table(time = 1:50, projection = c(rep("k", 5), rep("m", 14), rep("l", 31)), name = rep(c("CHF 2Y", "CHF 10Y", "CHF 30Y"), times = c(5, 14, 31)))
  expect_identical(getRateWithTime(marketRiskFactorsRate, maturitiesInterestRate), RateWithTime_result)
})

# test_that("process", {
# 
#   # Data remains unchanged
#   expect_identical(process(1L, main = TRUE, resultsForShinyDashboard = FALSE, debugMode = TRUE), 1L)
#   expect_identical(process(1L, main = TRUE, resultsForShinyDashboard = FALSE, debugMode = FALSE), 1L)
#   expect_identical(process(list(LETTERS), main = TRUE, resultsForShinyDashboard = FALSE, debugMode = FALSE), list(LETTERS))
#   
#   # Check option 'resultsForShinyDashboard'
#   output <- finalizeProzess(1L, packageEnv$errorLog, resultsForShinyDashboard = TRUE)
#   expect_identical(process(1L, main = TRUE, resultsForShinyDashboard = TRUE, debugMode = FALSE), output)
#   
#   # Environment remains unchanged
#   packageEnv$config <- 1:10
#   process("x", main = FALSE, resultsForShinyDashboard = FALSE, debugMode = FALSE)
#   expect_identical(packageEnv$config, 1:10)
#   
#   packageEnv$config <- 1:10
#   process("x", main = TRUE, resultsForShinyDashboard = FALSE, debugMode = TRUE)
#   expect_identical(packageEnv$config, 1:10)
#   
#   # Environment ist correctly reset
#   packageEnv$config <- 1:10
#   process("x", main = TRUE, resultsForShinyDashboard = FALSE, debugMode = FALSE)
#   expect_null(packageEnv$config)
#   
#   # Bug check: Pb should be closed at end of process
#   cleanup()
#   expect_output(process(createProgressBar("test", 1), main = FALSE), "test")
#   expect_null(packageEnv$pb)
#   
#   
#   expect_identical(process(NA), NA)
#   
#   expect_error(process(stop("Error message")), "Error message")
#   expect_warning(process(warning("Warning message")), "Warning message")
#   expect_warning(process(addError(NA, "Warning message", type = "Warning")), "Warning message")
#   expect_error(process(addError(NA, "Error message", type = "Error")), "Error message")
# })
