context("Captives")

captiveCY_stochastic_Empty <- captiveCY_stochastic[integer(0)]
captiveCY_deterministic_Empty <- captiveCY_deterministic[integer(0)]
captivePY_Empty <- captivePY[integer(0)]
nonlifeSimulation_Empty <- nonlifeSimulation[integer(0)]
nonlifeCDF_Empty <- nonlifeCDF[integer(0)]

objects_Empty <- getNonLifeRisks("no nonlife risk", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty)
objects_Sim <- getNonLifeRisks("simulations", nonlifeSimulation, NA, NA, nonlifeCDF_Empty, -1000, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty)
objects_LN <- getNonLifeRisks("lognormal parameters", nonlifeSimulation_Empty, 1, 2, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty)
objects_cdf <- getNonLifeRisks("cumulative distribution function", nonlifeSimulation_Empty, NA, NA, nonlifeCDF, 0, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty)
objects_captive <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic[1:4], captivePY)

test_that("getNonLifeRisks (constructor)", {
  # Constructor
  y <- retClass(list(param=retClass(list(), "NL_none"), nonLifeSeedAddon=1), "nonlifeModule")
  expect_identical(objects_Empty, y)
  
  y <- retClass(list(param=retClass(list(simulation = c(4, 2, 3, 1)), "NL_sim"), nonLifeSeedAddon = -1000), "nonlifeModule")
  expect_identical(objects_Sim, y)
  
  y <- retClass(list(param=retClass(list(mu = 1, sigma = 2), "NL_log"), nonLifeSeedAddon = 1), "nonlifeModule")
  expect_identical(objects_LN, y)
  
  y <- retClass(list(param=retClass(list(cdf = data.table(x = -2:2, cdf = (1:5)/5)),"NL_cdf"), nonLifeSeedAddon = 0), "nonlifeModule")
  expect_identical(objects_cdf, y)
  
  y <- retClass(list(param=retClass(list(CY=retClass(list(stochastic=captiveCY_stochastic_Result, deterministic=95), "NL_captCY"), PY=retClass(list(stochastic=captivePY_Result), "NL_captPY")), "NL_capt"),nonLifeSeedAddon=0), "nonlifeModule")
  expect_equal(objects_captive, y)
})

test_that("getNonLifeRisks (warnings)", {
  # CDF
  expect_warning(getNonLifeRisks("cumulative distribution function", nonlifeSimulation, NA, NA, nonlifeCDF, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty),
                 "The input will be ignored as a different non-life type is selected (type selected: 'cumulative distribution function')", fixed=TRUE)
  
  expect_silent(getNonLifeRisks("cumulative distribution function", nonlifeSimulation_Empty, NA, NA, nonlifeCDF, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty))
  
  # Simulations
  expect_warning(getNonLifeRisks("simulations", nonlifeSimulation, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic, captiveCY_stochastic_Empty, captivePY_Empty),
                 "The input will be ignored as a different non-life type is selected (type selected: 'simulations')", fixed=TRUE)
  expect_warning(getNonLifeRisks("simulations", nonlifeSimulation, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic, captivePY_Empty),
                 "The input will be ignored as a different non-life type is selected (type selected: 'simulations')", fixed=TRUE)
  expect_warning(getNonLifeRisks("simulations", nonlifeSimulation, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY),
                 "The input will be ignored as a different non-life type is selected (type selected: 'simulations')", fixed=TRUE)
  expect_silent(getNonLifeRisks("simulations", nonlifeSimulation, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty))
  
  # Captive
  expect_warning(getNonLifeRisks("captive", nonlifeSimulation_Empty, 1, 3.2, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic[1:4], captivePY_Empty),
                 "The input will be ignored as a different non-life type is selected (type selected: 'captive')", fixed=TRUE)
  
  expect_silent(getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic[1:4], captivePY_Empty))
  expect_silent(getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic_Empty, captiveCY_stochastic[1:4], captivePY_Empty))
  expect_silent(getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic_Empty, captivePY_Empty))
  expect_silent(getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY))
  
})

test_that("getNonLifeRisks (errors)", {
  # Errors
  expect_error(getNonLifeRisks("simulations", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty),
               "Please ensure that the input for the risk type 'simulations' is provided or set the risk type to 'no nonlife risk' if no non-life risk should be simulated", fixed=TRUE)

  expect_error(getNonLifeRisks("cumulative distribution function", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty),
               "Please ensure that the input for the risk type 'cumulative distribution function' is provided or set the risk type to 'no nonlife risk' if no non-life risk should be simulated", fixed=TRUE)
  
  expect_error(getNonLifeRisks("lognormal parameters", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty),
               "Please ensure that the input for the risk type 'lognormal parameters' is provided or set the risk type to 'no nonlife risk' if no non-life risk should be simulated", fixed=TRUE)
  
  expect_error(getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 1, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY_Empty),
               "Please ensure that the input for the risk type 'captive' is provided or set the risk type to 'no nonlife risk' if no non-life risk should be simulated", fixed=TRUE)
  
})


test_that("simulate.nonlifeModule (no captive)",{
  # No risk
  x <- simulate.nonlifeModule(objects_Empty, nsim = 10, seed = 1)
  y <- list(sim = data.table(all = rep(0, 10), CY = 0, PY = 0), RF = NULL)
  expect_identical(x, y)
  
  # Without replacement
  x <- simulate.nonlifeModule(object = objects_Sim, nsim = 3, seed = 1)
  setSeed(1, "NonLife", -1000)
  y <- list(sim = data.table(all = sample(x = c(4, 2, 3, 1), replace = FALSE, size = 3), CY=0, PY=0), RF = NULL)
  expect_identical(x, y)

  # Limit case
  x <- simulate.nonlifeModule(object = objects_Sim, nsim = 4, seed = 1)
  setSeed(1, "NonLife", -1000)
  y <- list(sim = data.table(all = sample(x = c(4, 2, 3, 1), replace = FALSE, size = 4), CY=0, PY=0), RF = NULL)
  expect_identical(x, y)
  
  # Limit case
  x <- simulate.nonlifeModule(object = objects_Sim, nsim = 5, seed = 1)
  setSeed(1, "NonLife", -1000)
  y <- list(sim = data.table(all = sample(x = c(4, 2, 3, 1), replace = TRUE, size = 5), CY=0, PY=0), RF = NULL)
  expect_identical(x, y)
  
  # Lognormal
  x <- simulate.nonlifeModule(object = objects_LN, nsim = 3, seed = 1)
  setSeed(1, "NonLife", 1)
  sim <- (stats::rnorm(n = 3) * 2 + 1)
  y <- list(sim = data.table(all = -(exp(sim) - exp(1 + (2^2)/2)), CY=0, PY=0), RF = NULL)
  expect_identical(x, y)
  
  # CDF
  x <- simulate.nonlifeModule(object = objects_cdf, nsim = 4, seed = 1)
  setSeed(1, "NonLife")
  y <- list(sim = data.table(all = sample(x = -2:2, size = 4, prob = diff(c(0, 0.2, 0.4, 0.6, 0.8, 1)), replace = T), CY=0, PY=0), RF = NULL)
  expect_identical(x, y)
})


objects_captiveCYdet <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic_Empty, captivePY_Empty)
objects_captiveCYstoch <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic_Empty, captiveCY_stochastic[1:2], captivePY_Empty)
objects_captiveCYboth <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic[1:2], captivePY_Empty)
objects_captivePY <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic_Empty, captiveCY_stochastic_Empty, captivePY)
objects_captiveCYdetPY <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic_Empty, captivePY)
objects_captiveCYbothPY <- getNonLifeRisks("captive", nonlifeSimulation_Empty, NA, NA, nonlifeCDF_Empty, 0, captiveCY_deterministic[1:3], captiveCY_stochastic[1:2], captivePY)


test_that("simulate.NL_captCY",{
  # Deterministic
  x <- simulate.NL_captCY(objects_captiveCYdet$param$CY, nsim=3)
  y <- rep(-95, 3)
  expect_identical(x, y)
  
  # Stochastic (small numerical error therefore check for equality instead of identical)
  set.seed(1)
  x <- simulate.NL_captCY(objects_captiveCYstoch$param$CY, nsim=3)
  set.seed(1)
  y <- simulateLineOfBusinessCY(nsim=3, captiveCY_stochastic_Result[1]) + simulateLineOfBusinessCY(nsim=3, captiveCY_stochastic_Result[2]) 
  expect_equal(x, y)
  
  # Both
  set.seed(1)
  x <- simulate.NL_captCY(objects_captiveCYboth$param$CY, nsim=3)
  set.seed(1)
  y <- rep(-95, 3) + simulateLineOfBusinessCY(nsim=3, captiveCY_stochastic_Result[1]) + simulateLineOfBusinessCY(nsim=3, captiveCY_stochastic_Result[2]) 
  expect_identical(x, y)
  
  # None
  x <- simulate.NL_captCY(objects_captivePY$param$CY, nsim=3)
  y <- rep(0, 3)
  expect_identical(x, y)
})
  

test_that("simulate.NL_captPY", {
  # PY Risk
  set.seed(1)
  x <- simulate.NL_captPY(objects_captivePY$param$PY, nsim=3)
  set.seed(1)
  y <- (simulateLineOfBusinessPY(nsim=3, captivePY_Result[1]) + simulateLineOfBusinessPY(nsim=3, captivePY_Result[2]))
  expect_identical(x, y)

  # Make sure that x is sorted
  expect_identical(x, sort(x))

  # No PY risk
  x <- simulate.NL_captPY(objects_captiveCYboth$param$PY, nsim=3)
  y <- rep(0, 3)
  expect_identical(x, y)
})

test_that("simulate.NL_capt", {
  # PY only
  set.seed(1)
  x <- simulate.NL_capt(objects_captivePY$param, nsim = 3)
  set.seed(1)
  u <- simulate.NL_captPY(objects_captivePY$param$PY, nsim = 3)[sample.int(3)]
  y <- data.table(all = u, CY = 0, PY = u)
  expect_identical(x,y)
  
  # CYdet only
  x <- simulate.NL_capt(objects_captiveCYdet$param, nsim = 3)
  u <- rep(-95, 3)
  y <- data.table(all = u, CY = u, PY = 0)
  expect_identical(x,y)
  
  # CYstoch only
  set.seed(1)
  x <- simulate.NL_capt(objects_captiveCYstoch$param, nsim = 3)
  set.seed(1)
  u <- sort(simulate.NL_captCY(objects_captiveCYstoch$param$CY, nsim = 3))[sample.int(3)]
  y <- data.table(all = u, CY = u, PY = 0)
  expect_identical(x,y)
  
  # CY and PY 
  set.seed(1)
  x <- simulate.NL_capt(objects_captiveCYbothPY$param, nsim = 3)
  set.seed(1)
  u <- sort(simulate.NL_captCY(objects_captiveCYbothPY$param$CY, nsim = 3))
  v <- sort(simulate.NL_captPY(objects_captiveCYbothPY$param$PY, nsim = 3))
  y <- data.table(all = u + v, CY = u, PY = v)[sample.int(3)]
  expect_identical(x,y)
})
