context("Variant")

variantNames <- c("Base case" = "", "Variant V1" = "V1", "Variant V2" =  "V2", "Variant V3" = "V3")
test_that("getVariant", {
  path <- system.file("extdata", c("Testfile.xlsx", "TestfileVariant.xlsx"), package="sstCalculation")
  
  # No variant
  x <- getVariant(path[1])
  y <- variantNames[1]
  expect_identical(x, y)
  
  # Variant
  x <- getVariant(path[2])
  y <- variantNames
  expect_identical(x, y)
  
  # Variant
  x <- getVariant(path)
  y <- variantNames
  expect_identical(x, y)
})


test_that("getVariantFromSheets", {
  sheetsList <- list(a = c("A", "B", "Test", "ABC__V3"), b = list("A__V1", "X__V2"), c = list("Z"))
  
  # Classical case
  x <- getVariantFromSheets(sheetsList)
  y <- variantNames
  expect_identical(x, y)

  # No variant
  x <- getVariantFromSheets(sheetsList[3])
  y <- variantNames[1]
  expect_identical(x, y)
  
  # One variant
  x <- getVariantFromSheets(sheetsList[1])
  y <- variantNames[c(1,4)]
  expect_identical(x, y)
})

test_that("loadExcelSheet", {
  path <- system.file("extdata", "Testfile.xlsx", package="sstCalculation")
  excelSheet <- list(A = list(sheet = "Sheet A", maxCol = 1),
                     B = list(sheet = "Sheet A", maxCol = 3),
                     C = list(sheet = "Sheet B", maxCol = 2),
                     D = list(sheet = "Sheet C", maxCol = 2)
  )
  
  # Ignore additional columns
  x <- loadExcelSheet(excelSheet$A, NULL, path)
  y <- matrix(as.character(1:3), ncol = 1)
  expect_identical(x, y)
  
  # Load an additional column beyond the data range in Excel
  x <- loadExcelSheet(excelSheet$B, NULL, path)
  y <- matrix(c("1", "2", "3", NA, NA, NA, "5", "6", rep(NA, 4)), ncol = 3)
  expect_identical(x, y)
  
  # Empty worksheet
  # Note: It may be better to produce a 3x0 matrix instead of a 0x0 matrix
  x <- loadExcelSheet(excelSheet$C, NULL, path)
  y <- matrix(nrow = 0, ncol = 0)
  expect_identical(x, y)
  
  # Load an additional column beyond the data range in Excel
  x <- loadExcelSheet(excelSheet$D, NULL, path)
  y <- matrix(c("Merged1", NA, NA, NA, "Merged", "A"), ncol = 2)
  expect_identical(x, y)
})